# noinspection PyMethodOverriding,PyArgumentList,PyPep8Naming
class classproperty(property):
    def __init__(self, fget, *arg, **kw):
        super(classproperty, self).__init__(fget, *arg, **kw)
        self.__doc__ = fget.__doc__

    def __get__(desc, self, cls):
        return desc.fget(cls)
